package fr.ippon.kafkaLatency.streaming;

import com.fasterxml.jackson.databind.ObjectMapper;
import fr.ippon.kafkaLatency.model.Person;
import fr.ippon.kafkaLatency.utils.Config;
import fr.ippon.kafkaLatency.utils.UtilStats;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.log4j.Logger;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.function.Function;
import org.apache.spark.api.java.function.VoidFunction;
import org.apache.spark.streaming.Durations;
import org.apache.spark.streaming.api.java.JavaInputDStream;
import org.apache.spark.streaming.api.java.JavaStreamingContext;
import org.apache.spark.streaming.kafka010.ConsumerStrategies;
import org.apache.spark.streaming.kafka010.KafkaUtils;
import org.apache.spark.streaming.kafka010.LocationStrategies;

import java.io.IOException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * Ippon Tech 2018.
 */
public class KafkaSpark {
    static ObjectMapper mapper = new ObjectMapper();

    private final static Logger logger = Logger.getLogger(KafkaContinuousSpark.class);

    static Function function = (Function<ConsumerRecord<String, String>, Person>) cr -> {
            LocalDateTime kafkaTimestamp = LocalDateTime.ofInstant(Instant.ofEpochMilli(cr.timestamp()), TimeZone
                    .getDefault().toZoneId());
            LocalDateTime now = LocalDateTime.now();

            long millis = ChronoUnit.MILLIS.between(kafkaTimestamp, now);
            Person p = mapper.readValue(cr.value(), Person.class);

            UtilStats.treatStats(millis);

        return p;

    };

    public static void main(String[] args) {
        JavaStreamingContext sc = null;
        try {
            String brokers = Config.getProperty("bootstrap.servers");
            String topic = Config.getProperty("topic");
            Integer SparkStreamingInterval = Config.getIntProperty("spark.streaming.interval");
            Integer maxPollRecords = Config.getIntProperty("max.poll.records");

            SparkConf sparkConf = new SparkConf()
                    .setMaster("local[*]")
                    .set("spark.streaming.stopGracefullyOnShutdown", "true")
                    .set("spark.ui.enabled", "false")
                    .set("spark.eventLog.enabled", "false")
                    .set("spark.streaming.backpressure.enabled", "true")
                    .setAppName("kafkaSparkStreaming");

            sc = new JavaStreamingContext(sparkConf, Durations.milliseconds(SparkStreamingInterval));


            Map<String, Object> kafkaParams = new HashMap<>();

            kafkaParams.put("bootstrap.servers", brokers);
            kafkaParams.put("key.deserializer", StringDeserializer.class);
            kafkaParams.put("value.deserializer", StringDeserializer.class);
            kafkaParams.put("group.id", "KafkaSpark");
            kafkaParams.put("auto.offset.reset", "latest");
            kafkaParams.put("max.poll.records", maxPollRecords);

            Collection<String> topics = Arrays.asList(topic);

            final JavaInputDStream<ConsumerRecord<String, String>> stream =
                    KafkaUtils.createDirectStream(
                            sc,
                            LocationStrategies.PreferConsistent(),
                            ConsumerStrategies.<String, String>Subscribe(topics, kafkaParams)
                    );

            stream.foreachRDD((VoidFunction<JavaRDD<ConsumerRecord<String, String>>>) rdd -> rdd.map(function).count());

            sc.start();
            sc.awaitTermination();

        } catch (Exception e) {
            logger.error("Error " + e.getMessage());
            sc.stop(true, true);
        }
    }

}






